@@ -14,7 +14,7 @@ my $prefix = __PACKAGE__;
 # this module import config
 #
 use Carp ();
-use Net::SNMP::Mixin::Util qw/normalize_mac idx2val push_error/;
+use Net::SNMP::Mixin::Util qw/normalize_mac idx2val push_error get_init_slot/;
 
 #
 # this module export config
@@ -74,7 +74,7 @@ Net::SNMP::Mixin::Dot1abLldp - mixin class for the Link Layer Discovery Protocol
 
 =cut
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 =head1 SYNOPSIS
 
@@ -90,7 +90,7 @@ A Net::SNMP mixin class for LLDP (Link Layer Discovery Protocol) based info.
   $session->mixer('Net::SNMP::Mixin::Dot1abLldp');
   $session->init_mixins;
   snmp_dispatcher();
-
+  $session->init_ok();
   die $session->errors if $session->errors;
 
   printf "Local ChassisID: %s\n",
@@ -140,8 +140,10 @@ Returns the LLDP lldpLocalSystemData group as a hash reference:
 
 sub get_lldp_local_system_data {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # just a shallow copy for shallow values
   my $result = { %{ $session->{$prefix}{locSysData} } };
@@ -185,8 +187,10 @@ See also the L<< Net::SNMP::Mixin::Dot1dBase >> for a mixin to get the mapping b
 
 sub get_lldp_loc_port_table {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # stash for return values
   my $result = {};
@@ -239,8 +243,10 @@ Returns the LLDP lldp_rem_table as a hash reference. The table is indexed by the
 
 sub get_lldp_rem_table {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
 
   # stash for return values
   my $result = {};
@@ -289,11 +295,23 @@ Fetch the LLDP related snmp values from the host. Don't call this method direct!
 
 =cut
 
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 3;
+
 sub _init {
   my ($session, $reload) = @_;
+  my $agent = $session->hostname;
 
-  die "$prefix already initalized and reload not forced.\n"
-  	if $session->{$prefix}{__initialized} && not $reload;
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+      && get_init_slot($session)->{$prefix} == 0
+      && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # populate the object with needed mib values
   #
@@ -334,17 +352,14 @@ sub _fetch_lldp_local_system_data {
     -endindex   => '6.0', # LLDP_LOCAL_SYS_CAPA_ENA
 
     # define callback if in nonblocking mode
-    $session->nonblocking
-    ? ( -callback => \&_lldp_local_system_data_cb )
-    : (),
+    $session->nonblocking ? ( -callback => \&_lldp_local_system_data_cb ) : (),
 
   );
 
   unless (defined $result) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -367,10 +382,9 @@ sub _lldp_local_system_data_cb {
   my $vbl     = $session->var_bind_list;
 
   unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -392,7 +406,10 @@ sub _lldp_local_system_data_cb {
   $session->{$prefix}{locSysData}{lldpLocSysCapEnabled} =
     $vbl->{ LLDP_LOCAL_SYS_CAPA_ENA() };
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _fetch_lldp_loc_port_tbl($session) >>
@@ -410,20 +427,13 @@ sub _fetch_lldp_loc_port_tbl {
     -baseoid => LLDP_LOC_PORT_TABLE,
 
     # define callback if in nonblocking mode
-    $session->nonblocking
-    ? ( -callback => \&_lldp_loc_port_tbl_cb )
-    : (),
-
-    # dangerous for snmp version 2c and 3,
-    # some agents are very buggy, like ExtremeNetworks Ver. 7.7.1
-    $session->version ? ( -maxrepetitions => 0 ) : (),
+    $session->nonblocking ? ( -callback => \&_lldp_loc_port_tbl_cb ) : (),
   );
 
   unless (defined $result) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -446,11 +456,11 @@ sub _lldp_loc_port_tbl_cb {
   my $session = shift;
   my $vbl     = $session->var_bind_list;
 
+
   unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -465,7 +475,10 @@ sub _lldp_loc_port_tbl_cb {
   $session->{$prefix}{lldpLocPortTbl}{lldpLocPortDesc} =
     idx2val( $vbl, LLDP_LOC_PORT_DESC );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 =head2 B<< _fetch_lldp_rem_tbl($session) >>
@@ -483,20 +496,13 @@ sub _fetch_lldp_rem_tbl {
     -baseoid => LLDP_REM_TABLE,
 
     # define callback if in nonblocking mode
-    $session->nonblocking
-    ? ( -callback => \&_lldp_rem_tbl_cb )
-    : (),
-
-    # dangerous for snmp version 2c and 3,
-    # some agents are very buggy, like ExtremeNetworks Ver. 7.7.1
-    $session->version ? ( -maxrepetitions => 0 ) : (),
+    $session->nonblocking ? ( -callback => \&_lldp_rem_tbl_cb ) : (),
   );
 
   unless (defined $result) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -520,10 +526,9 @@ sub _lldp_rem_tbl_cb {
   my $vbl     = $session->var_bind_list;
 
   unless (defined $vbl) {
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error($session, "$prefix: $err_msg") if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    }
     return;
   }
 
@@ -591,7 +596,10 @@ sub _lldp_rem_tbl_cb {
   $session->{$prefix}{lldpRemTbl}{lldpRemSysCapEnabled} =
     idx2val( $vbl, LLDP_REM_SYS_CAPA_ENA, 1, undef, );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 unless ( caller() ) {
@@ -621,7 +629,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2008-2012 Karl Gaissmaier, all rights reserved.
+Copyright 2008-2016 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
